'use strict';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function asyncMapInChunks(operationCount, asyncMap, batchSize = 20, onBatch = () => { }) {
    return __awaiter(this, void 0, void 0, function* () {
        const batchCount = Math.ceil(operationCount / batchSize);
        let result = [];
        for (let i = 0; i < batchCount; i += 1) {
            const currentBatchOperations = [];
            for (let j = 0; j < batchSize; j += 1) {
                const operationIndex = i * batchSize + j;
                if (operationIndex >= operationCount) {
                    break;
                }
                currentBatchOperations.push(asyncMap(operationIndex));
            }
            const currentBatchOperationsResults = yield Promise.all(currentBatchOperations);
            yield onBatch(i, batchCount);
            result = result.concat(currentBatchOperationsResults);
        }
        return result;
    });
}

module.exports = asyncMapInChunks;
